/***************************************************************************
                          htl_bounding_box.cpp  -  description
                             -------------------
    begin                : Sat Oct 4 2003
    copyright            : (C) 2003 by Soloviev Alexei,JINR,LIT-303,64713
    email                : solovjev@cv.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "htl_bounding_box.h"
#include "htl_text_label.h"
#include "gl2ps.h"

#include <cmath>
#include <cfloat>
#include <algorithm>
#include <strstream>
#include <string>
using namespace std;

#include <GL/glut.h>

htl::bounding_box::bounding_box () :
  _x_min(), _x_max(), _y_min(), _y_max(), 
  _is_log_x(false), _is_log_y(false), 
  _show_grid(true), _show_ticks(true),
  _title(), _x_label(), _y_label(),
  _x_min_offset(0.16),
  _x_max_offset(0.02),
  _y_min_offset(0.08),
  _y_max_offset(0.02)
{}

const double htl::bounding_box::xmin() const
{
  return ( logX() ? log10 ( max ( _x_min, epsilon ) ) : _x_min ) - xoffset();
}

const double htl::bounding_box::xmax() const
{
  return ( logX() ? log10 ( max ( _x_max, epsilon ) ) : _x_max ) + xoffset();
}

const double htl::bounding_box::ymin() const
{
  return ( logY() ? log10 ( max ( _y_min, epsilon ) ) : _y_min ) - yoffset();
}

const double htl::bounding_box::ymax() const
{
  return ( logY() ? log10 ( max ( _y_max, epsilon ) ) : _y_max ) + yoffset();
}

const double htl::bounding_box::xoffset() const
{
  return logX() ? 
    ( log10 ( max ( _x_max, epsilon ) ) - log10 ( max ( _x_min, epsilon ) ) ) / 20 : 
    ( _x_max - _x_min ) / 20;
}

const double htl::bounding_box::yoffset() const
{
  return logY() ? 
    ( log10 ( max ( _y_max, epsilon ) ) - log10 ( max ( _y_min, epsilon ) ) ) / 20 : 
    ( _y_max - _y_min ) / 20;
}

void htl::bounding_box::set_x_range ( const double & xmin, const double & xmax )
{
  if ( ( _x_min = xmin ) > ( _x_max = xmax ) ) swap ( _x_min, _x_max );
}

void htl::bounding_box::set_y_range ( const double & ymin, const double & ymax )
{
  if ( ( _y_min = ymin ) > ( _y_max = ymax ) ) swap ( _y_min, _y_max );
}

void htl::bounding_box::zoom ( const double & xmin, const double & xmax, 
			       const double & ymin, const double & ymax )
{
  _x_min = min ( _x_min, xmin );
  _x_max = max ( _x_max, xmax );
  _y_min = min ( _y_min, ymin );
  _y_max = max ( _y_max, ymax );
}

void htl::bounding_box::unzoom ( const double & xmin, const double & xmax, 
				 const double & ymin, const double & ymax )
{
  _x_min = max ( _x_min, xmin );
  _x_max = min ( _x_max, xmax );
  _y_min = max ( _y_min, ymin );
  _y_max = min ( _y_max, ymax );
}

void htl::bounding_box::shift ( const double & dx, const double & dy )
{
  _x_min += dx;
  _x_max += dx;
  _y_min += dy;
  _y_max += dy;
}

htl::bounding_box & htl::bounding_box::set_title ( const string & text )
{
  if ( _title.text().empty() ) _y_max_offset += 0.08;
  _title.text() = text;
  return *this;
}

htl::bounding_box & htl::bounding_box::set_x_label ( const string & text )
{
  if ( _x_label.text().empty() ) _y_min_offset += 0.06;
  _x_label.text() = text;
  return *this;
}

htl::bounding_box & htl::bounding_box::set_y_label ( const string & text )
{
  if ( _y_label.text().empty() ) _x_min_offset += 0.06;
  _y_label.text() = text;
  return *this;
}

void htl::bounding_box::plot() const
{
  glMatrixMode ( GL_PROJECTION );
  glLoadIdentity ();
  gluOrtho2D ( -1-_x_min_offset, 1+_x_max_offset, 
	       -1-_y_min_offset, 1+_y_max_offset );

  glMatrixMode ( GL_MODELVIEW );
  glLoadIdentity ();
  gluOrtho2D ( xmin(), xmax(), ymin(), ymax() );

  const double x_min_clip_plane[] = {1,0,0,-xmin()};
  const double x_max_clip_plane[] = {-1,0,0,xmax()};
  const double y_min_clip_plane[] = {0,1,0,-ymin()};
  const double y_max_clip_plane[] = {0,-1,0,ymax()};

  glClipPlane ( GL_CLIP_PLANE0, x_min_clip_plane );
  glClipPlane ( GL_CLIP_PLANE1, x_max_clip_plane );
  glClipPlane ( GL_CLIP_PLANE2, y_min_clip_plane );
  glClipPlane ( GL_CLIP_PLANE3, y_max_clip_plane );

  glDisable ( GL_CLIP_PLANE0 );
  glDisable ( GL_CLIP_PLANE1 );
  glDisable ( GL_CLIP_PLANE2 );
  glDisable ( GL_CLIP_PLANE3 );

  plot_axis ();
  if ( showGrid() ) plot_grid ();
  if ( showTicks() ) plot_ticks ();

  _title.plot();
  _x_label.plot();
  _y_label.plot();

  glEnable ( GL_CLIP_PLANE0 );
  glEnable ( GL_CLIP_PLANE1 );
  glEnable ( GL_CLIP_PLANE2 );
  glEnable ( GL_CLIP_PLANE3 );

  glutPostRedisplay ();
}

void htl::bounding_box::plot_axis() const
{
  // color
  glColor3ub ( 0, 0, 0 );

  // line
  glDisable ( GL_LINE_SMOOTH );
  glLineWidth ( 1.5 );
  gl2psLineWidth ( 1.5 );

  glBegin ( GL_LINE_LOOP );
    glVertex2f ( xmax(), ymin() );
    glVertex2f ( xmin(), ymin() );
    glVertex2f ( xmin(), ymax() );
    glVertex2f ( xmax(), ymax() );
  glEnd ();
}

void htl::bounding_box::plot_grid() const
{
  glColor3f ( 0.4, 0.4, 0.4 );

  glLineWidth ( 0.7 );
  gl2psLineWidth ( 0.7 );

  glDisable ( GL_LINE_SMOOTH );

  glEnable ( GL_LINE_STIPPLE );
  gl2psEnable ( GL2PS_LINE_STIPPLE );
  glLineStipple (1, 0x1111);

  double dx = x_tick_step();
  double x = dx * floor ( xmin() / dx );
  double dy = y_tick_step();
  double y = dy * floor ( ymin() / dy );

  if ( x < xmin() ) x += dx;
  while ( x < xmax() ) {
    glBegin ( GL_LINES );
    glVertex2f ( x, ymin() );
    glVertex2f ( x, ymax() );
    glEnd ();
    x += dx;
  }

  if ( y < ymin() ) y += dy;
  while ( y < ymax() ) {
    glBegin ( GL_LINES );
    glVertex2f ( xmin(), y );
    glVertex2f ( xmax(), y );
    glEnd ();
    y += dy;
  }

  glDisable ( GL_LINE_STIPPLE );
  gl2psDisable ( GL2PS_LINE_STIPPLE );
}

void htl::bounding_box::plot_ticks() const
{
  glColor3f ( 0, 0, 0 );
  glLineWidth ( 1.5 );
  gl2psLineWidth ( 1.5 );

  double dx = x_tick_step();
  double x = dx * floor ( xmin() / dx );
  double dy = y_tick_step();
  double y = dy * floor ( ymin() / dy );

  static strstream buff;
  static text_label tick;

  if ( x < xmin() ) x += dx;
  tick.y() = ymin();
  tick.set_x_alignment(text_label::center);
  tick.set_y_alignment(text_label::top);
  while ( x < xmax() ) {
    buff.clear();
    buff << ( logX() ? pow ( 10, x ) : x );
    buff>>tick.text();
    tick.x() = x;
    tick.plot();
    x += dx;
    if ( fabs(x) < epsilon ) x = 0;
  }

  if ( y < ymin() ) y += dy;
  tick.x() = xmin();
  tick.set_x_alignment(text_label::right);
  tick.set_y_alignment(text_label::middle);
  while ( y < ymax() ) {
    buff.clear();
    buff << ( logY() ? pow ( 10, y ) : y );
    buff>>tick.text();
    tick.y() = y;
    tick.plot();
    y += dy;
    if ( fabs(y) < epsilon ) y = 0;
  }
}

const double htl::bounding_box::tick_step ( const double & range ) const
{
  /* the guide parameter was intended to allow the number of tics
   * to depend on the relative sizes of the plot and the font.
   * It is the approximate upper limit on number of tics allowed.
   * But it did not go down well with the users.
   * A value of 20 gives the same behaviour as 3.5, so that is
   * hardwired into the calls to here. Maybe we will restore it
   * to the automatic calculation one day
   */
  static const int guide = 20;

  double l10 = log10(fabs(range));
  int fl = static_cast<int>(floor(l10));
  double decs = pow(10.0, l10 - fl);  /* approx number of decades */
  double posns = guide / decs;  /* approx number of tic posns per decade */

  static double tics;
  if (posns > 40) tics = 0.05;      /* eg 0, 0.05, 0.10, ... */
  else if (posns > 20) tics = 0.1;  /* eg 0, 0.1, 0.2, ... */
  else if (posns > 10) tics = 0.2;  /* eg 0, 0.2, 0.4, ... */
  else if (posns > 4) tics = 0.5;   /* eg 0, 0.5, 1, ... */
  else if (posns > 1) tics = 1;     /* eg 0, 1, 2, ... */
  else if (posns > 0.5) tics = 2;   /* eg 0, 2, 4, ... */
  else
    /* getting desperate... the ceil is to make sure we
     * go over rather than under - eg plot [-10:10] x*x
     * gives a range of about 99.999 - tics=xnorm gives
     * tics at 0, 99.99 and 109.98  - BAD !
     * This way, inaccuracy the other way will round
     * up (eg 0->100.0001 => tics at 0 and 101
     * I think latter is better than former
     */
    tics = ceil(decs);

  return (tics * pow(10.0, fl));
}

const double htl::bounding_box::x_tick_step() const
{
  double step = tick_step ( xmax() - xmin() );
  if ( logX() && step < 1.0 ) step = 1.0;
  return step;
}

const double htl::bounding_box::y_tick_step() const
{
  double step = tick_step ( ymax() - ymin() );
  if ( logY() && step < 1.0 ) step = 1.0;
  return step;
}
