/***************************************************************************
                          htl_bounding_box.h  -  description
                             -------------------
    begin                : Sat Oct 4 2003
    copyright            : (C) 2003 by Soloviev Alexei,JINR,LIT-303,64713
    email                : solovjev@cv.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HTL_BOUNDING_BOX_H
#define HTL_BOUNDING_BOX_H

#include "htl_text_label.h"

namespace htl {

static const double epsilon = 1e-5;

class bounding_box {

public:
  bounding_box ();
  virtual ~bounding_box () {}

  virtual const double xmin() const;
  virtual const double xmax() const;
  virtual const double ymin() const;
  virtual const double ymax() const;

  virtual const double xoffset() const;
  virtual const double yoffset() const;

protected:
  double _x_min, _x_max;
  double _y_min, _y_max;

public:
  virtual void set_x_range ( const double & xmin, const double & xmax );
  virtual void set_y_range ( const double & ymin, const double & ymax );
  virtual void zoom ( const double & xmin, const double & xmax, 
		      const double & ymin, const double & ymax );
  virtual void unzoom ( const double & xmin, const double & xmax, 
			const double & ymin, const double & ymax );
  virtual void shift ( const double & dx, const double & dy );

public:
  const bool & logX() const { return _is_log_x; }
  bool & logX() { return _is_log_x; }
  const bool & logY() const { return _is_log_y; }
  bool & logY() { return _is_log_y; }

protected:
  bool _is_log_x;
  bool _is_log_y;

public:
  const bool & showGrid() const { return _show_grid; }
  bool & showGrid() { return _show_grid; }
  const bool & showTicks() const { return _show_ticks; }
  bool & showTicks() { return _show_ticks; }

protected:
  bool _show_grid;
  bool _show_ticks;

public:
  virtual bounding_box & set_title ( const string & text );
  virtual bounding_box & set_x_label ( const string & text );
  virtual bounding_box & set_y_label ( const string & text );

protected:
  title _title;
  x_label _x_label;
  y_label _y_label;

public:
  virtual void plot() const;

protected:
  virtual void plot_axis() const;
  virtual void plot_grid() const;
  virtual void plot_ticks() const;

private:
  const double tick_step ( const double & range ) const;
  const double x_tick_step() const;
  const double y_tick_step() const;
  
protected:
  double _x_min_offset;
  double _x_max_offset;
  double _y_min_offset;
  double _y_max_offset;
};

} /** namespace htl */

#endif // HTL_BOUNDING_BOX_H

// Local Variables:
// mode:C++
// End:
