/***************************************************************************
                          htl_contour.h  -  description
                             -------------------
    begin                : Sat Oct 4 2003
    copyright            : (C) 2003 by Soloviev Alexei,JINR,LIT-303,64713
    email                : solovjev@cv.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HTL_CONTOUR_H
#define HTL_CONTOUR_H

#include <histogram>

namespace htl {

class contour : public histogram<double,bin<double> >
{
public:
  template<class Tx, class Ty>
  explicit contour ( const histogram<Tx,Ty> & h );

  virtual ~contour () {}

  enum STYLE { lines,
	       points,
	       linespoints,
	       dots,
	       impulses,
	       steps,
	       boxes,
	       candlesticks,
	       errorbars };

  const STYLE & style () const { return _style; }
  STYLE & style () { return _style; }

  const bool & logX () const { return _is_log_x; }
  bool & logX () { return _is_log_x; }
  const bool & logY () const { return _is_log_y; }
  bool & logY () { return _is_log_y; }

  const double & xmin () const;
  const double & xmax () const;
  const double & ymin () const;
  const double & ymax () const;

  virtual void plot () const;

  static void reset_color() { _color_index = 0; }

protected:
  virtual void plot_lines () const;
  virtual void plot_points () const;
  virtual void plot_impulses () const;
  virtual void plot_steps () const;
  virtual void plot_boxes () const;
  virtual void plot_errorbars () const;

protected:
  struct rgb { unsigned int r, g, b; } _color;
  static const rgb _web_color_rgbs[96];
  static unsigned int _color_index;
  STYLE  _style;
  bool _is_log_x;
  bool _is_log_y;
};

/***************************************************************************
  Implementations
 ***************************************************************************/

template<class Tx, class Ty>
contour::contour ( const histogram<Tx,Ty> & h ) :
  histogram<double,bin<double> >(h),
  _color(_web_color_rgbs[_color_index]),
  _style(lines), 
  _is_log_x(false), _is_log_y(false)
{
  if ( ++_color_index > 95 ) reset_color();
}

} /** namespace htl */

#endif // HTL_CONTOUR_H

// Local Variables:
// mode:C++
// End:
