/***************************************************************************
                          htl_plot.cpp  -  description
                             -------------------
    begin                : Sat Oct 4 2003
    copyright            : (C) 2003 by Soloviev Alexei,JINR,LIT-303,64713
    email                : solovjev@cv.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "htl_plot.h"

#include <algorithm>
#include <functional>
#include <string>
#include <cstdio>
using namespace std;

#include <GL/glut.h>

htl::hplot::hplot ( const char* title,
		    int x,
		    int y,
		    int width,
		    int height ) :
  bounding_box(),
  vector<contour>(),
  _style(contour::lines),
  _show_background(false),
  _background_image(0),
  _do_mark(false),
  _do_move(false)
{
  if ( _registry.empty() ) _init();
  contour::reset_color();

  glutInitWindowPosition ( x, y );
  glutInitWindowSize ( width, height );

  static const unsigned int mode = GLUT_RGB | GLUT_DOUBLE;
  glutInitDisplayMode ( mode );

  _registry[ glutCreateWindow ( title ) ] = this;

  glutDisplayFunc ( _display );

  glutKeyboardFunc ( _keyboard );
  glutMouseFunc ( _mouse );
  glutMotionFunc ( _motion );

  _scale_menu = glutCreateMenu ( _menu );
  glutAddMenuEntry ( "[x]-axis: lin->log", 'x' );
  glutAddMenuEntry ( "[y]-axis: lin->log", 'y' );

  _style_menu = glutCreateMenu ( _menu );
  glutAddMenuEntry ( "lines", contour::lines );
  glutAddMenuEntry ( "points", contour::points );
  glutAddMenuEntry ( "linespoints", contour::linespoints );
  glutAddMenuEntry ( "dots", contour::dots );
  glutAddMenuEntry ( "impulses", contour::impulses );
  glutAddMenuEntry ( "steps", contour::steps );
  glutAddMenuEntry ( "boxes", contour::boxes );
  glutAddMenuEntry ( "candlesticks", contour::candlesticks );
  glutAddMenuEntry ( "errorbars", contour::errorbars );

  _decoration_menu = glutCreateMenu ( _menu );
  glutAddMenuEntry ( "[g]rid: yes", 'g' );
  glutAddMenuEntry ( "[t]icks: yes", 't' );
  glutAddMenuEntry ( "[b]ackground: no", 'b' );
  glutAddMenuEntry ( "[f]it to window", 'f' );

  glutCreateMenu ( _menu );
  glutAddSubMenu ( "Scale...", _scale_menu );
  glutAddSubMenu ( "Style...", _style_menu );
  glutAddSubMenu ( "Decoration...", _decoration_menu );
  glutAddMenuEntry ( "[S]ave EPS", 's' );
  glutAddMenuEntry ( "[Q]uit", 'q' );
  glutAttachMenu ( GLUT_RIGHT_BUTTON );
}

htl::hplot::~hplot ()
{
  glutMainLoop();
}

htl::hplot & htl::hplot::operator<< ( const contour & c )
{
  if ( empty() ) {
    set_x_range ( c.xmin(), c.xmax() );
    set_y_range ( c.ymin(), c.ymax() );
  }
  zoom ( c.xmin(), c.xmax(), c.ymin(), c.ymax() );
  push_back ( c );
  rbegin()->style() = style();
  return *this;
}

htl::hplot & operator<< ( htl::hplot & pout, htl::contour::STYLE style )
{
  pout.style() = style;
  return pout;
}

void htl::hplot::setup()
{
  const_iterator p = begin();
  set_x_range ( p->xmin(), p->xmax() );
  set_y_range ( p->ymin(), p->ymax() );
  while ( ++p != end() )
    zoom ( p->xmin(), p->xmax(), p->ymin(), p->ymax() );
}

void htl::hplot::display ()
{
  glClearColor ( 1.0f, 1.0f, 1.0f, 1.0f );
  glClear ( GL_COLOR_BUFFER_BIT );

  glMatrixMode ( GL_MODELVIEW );
  glLoadIdentity ();
  glMatrixMode ( GL_PROJECTION );
  glLoadIdentity ();

  if ( _show_background ) display_background();

  plot();

  iterator p = begin();
  while ( p != end() ) {
    p->logX() = logX();
    p->logY() = logY();
    p++->plot();
  }

  if ( _do_mark ) {
    glColor4f ( 0, 0, 0, 0.08 );
    glRectd ( _x_min_mark, _y_min_mark,
	      _x_max_mark, _y_max_mark );
  }

  glutSwapBuffers ();
}

void htl::hplot::keyboard ( unsigned char key,
			    int x,
			    int y )
{
  menu ( key );
  glutPostRedisplay ();
}

void htl::hplot::mouse ( int button, int state, int x, int y )
{
  if ( button == GLUT_LEFT_BUTTON ) {

    static double x0, y0;
    static double x1, y1;

    if ( state == GLUT_DOWN ) {

      x0 = _x_min_mark = xmin();
      x1 = _x_max_mark = xmax();
      y0 = _y_min_mark = ymin();
      y1 = _y_max_mark = ymax();

      unproject ( x, y, x0, y0 );

      if ( x0 > xmin() && x0 < xmax() && y0 < ymin() ) {
	_x_min_mark = _x_max_mark = x0;
	_do_mark = true;
      }

      if ( y0 > ymin() && y0 < ymax() && x0 < xmin() ) {
	_y_min_mark = _y_max_mark = y0;
	_do_mark = true;
      }

      if ( x0 < xmin() && y0 < ymin() ) {
	setup();
      }

    } else { // state == GLUT_UP

      unproject ( x, y, x1, y1 );

      if ( x1 > xmin() && x1 < xmax() && y1 < ymin() && 
	   fabs(x0-x1) > epsilon ) {
	if ( logX() ) {
	  x0 = pow ( 10, x0 );
	  x1 = pow ( 10, x1 );
	}
	set_x_range ( x0, x1 );
      }

      if ( y1 > ymin() && y1 < ymax() && x1 < xmin() && 
	   fabs(y0-y1) > epsilon ) {
	if ( logY() ) {
	  y0 = pow ( 10, y0 );
	  y1 = pow ( 10, y1 );
	}
	set_y_range ( y0, y1 );
      }

      _do_mark = false;
    }
  }

  if ( button == GLUT_MIDDLE_BUTTON ) {
    if ( state == GLUT_DOWN ) {
      _do_move = true;
    } else { // state == GLUT_UP
      _do_move = false;
    }
  }

  glutPostRedisplay ();
}

void htl::hplot::motion ( int x, int y )
{
  if ( _do_mark ) {
    static double xx, yy;
    unproject ( x, y, xx, yy );

    if ( xx > xmin() && xx < xmax() && yy < ymin() ) {
      _x_max_mark = xx;
    }
    if ( yy > ymin() && yy < ymax() && xx < xmin() ) {
      _y_max_mark = yy;
    }
  }

  if ( _do_move ) {
    static int xold;
    static int yold;

    float dx = ( _x_max - _x_min ) / 64;
    float dy = ( _y_max - _y_min ) / 64;
    dx *= x < xold ? 1 : x > xold ? -1 : 0;
    dy *= y > yold ? 1 : y < yold ? -1 : 0;

    if ( logX() && dx < -_x_max ) dx = 0;
    if ( logY() && dy < -_y_max ) dy = 0;

    shift ( dx, dy );

    xold = x;
    yold = y;
  }

  glutPostRedisplay ();
}

void htl::hplot::menu ( int option )
{
  switch ( option ) {

  // Scale...
  case 'x': logX() = !logX(); break;
  case 'y': logY() = !logY(); break;

  // Style...
  case contour::lines:
  case contour::points:
  case contour::linespoints:
  case contour::dots:
  case contour::impulses:
  case contour::steps:
  case contour::boxes:
  case contour::candlesticks:
  case contour::errorbars:
    style() = static_cast<contour::STYLE>(option);
    for ( iterator p = begin(); p != end(); ++p ) p->style() = style();
    break;

  // Decoration...
  case 'g':
    showGrid() = !showGrid();
    break;
  case 't':
    showTicks() = !showTicks();
    break;
  case 'b':
    _show_background = !_show_background;
    if ( _show_background ) load_background_image("marble.ppm");
    break;
  case 'f':
    setup();
    break;

  // Backgrounds...
  case '1':
    if ( _show_background ) load_background_image("marble.ppm");
    break;
  case '2':
    if ( _show_background ) load_background_image("ice.ppm");
    break;
  case '3':
    if ( _show_background ) load_background_image("wood.ppm");
    break;
  case '4':
    if ( _show_background ) load_background_image("paper.ppm");
    break;
  case '5':
    if ( _show_background ) load_background_image("burlwood.ppm");
    break;

  // ...
  case 's':
    save ( string(PACKAGE)+=".eps" );
    break;
  case 'q':
  case 27: /** Escape */
    exit(0);
    break;
  }

  // "Scale..." menu update
  glutSetMenu ( _scale_menu );
  if ( logX() ) {
    glutChangeToMenuEntry ( 1, "[x]-axis: log->lin", 'x' );
  } else {
    glutChangeToMenuEntry ( 1, "[x]-axis: lin->log", 'x' );
  }
  if ( logY() ) {
    glutChangeToMenuEntry ( 2, "[y]-axis: log->lin", 'y' );
  } else {
    glutChangeToMenuEntry ( 2, "[y]-axis: lin->log", 'y' );
  }

  // "Decoration..." menu update
  glutSetMenu ( _decoration_menu );
  if ( showGrid() ) {
    glutChangeToMenuEntry ( 1, "[g]rid: yes", 'g' );
  } else {
    glutChangeToMenuEntry ( 1, "[g]rid: no", 'g' );
  }
  if ( showTicks() ) {
    glutChangeToMenuEntry ( 2, "[t]icks: yes", 't' );
  } else {
    glutChangeToMenuEntry ( 2, "[t]icks: no", 't' );
  }
  if ( _show_background ) {
    glutChangeToMenuEntry ( 3, "[b]ackground: yes", 'b' );
  } else {
    glutChangeToMenuEntry ( 3, "[b]ackground: no", 'b' );
  }


  glutPostRedisplay ();
}

void htl::hplot::unproject ( int xw, int yw, double & x, double & y ) const
{
  static double modvmatrix[16];
  static double projmatrix[16];
  static int viewport[4];

  glGetDoublev ( GL_MODELVIEW_MATRIX, modvmatrix );
  glGetDoublev ( GL_PROJECTION_MATRIX, projmatrix );
  glGetIntegerv ( GL_VIEWPORT, viewport );

  static double zw;
  static double z;

  gluUnProject ( (double)xw, (double)(viewport[3]-yw-1), zw,
		 modvmatrix, projmatrix, viewport, 
		 &x, &y, &z );
}

extern "C" {
#include "glm.h"
}

bool htl::hplot::load_background_image ( const string & name )
{
  unsigned char* _image_back = _background_image;

  _background_image = glmReadPPM ( const_cast<char*>(name.c_str()), 
				   &_background_image_width, 
				   &_background_image_height );
  if ( _background_image ) {
    gluBuild2DMipmaps ( GL_TEXTURE_2D, 
			3, 
			_background_image_width, 
			_background_image_height, 
			GL_RGB, 
			GL_UNSIGNED_BYTE, 
			_background_image );
    free ( _image_back );
    return true;
  }

  _background_image = _image_back;
  return false;
}

void htl::hplot::display_background()
{
  if ( _background_image ) {
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
    glTexEnvi ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );

    glEnable ( GL_TEXTURE_2D );
    glEnable ( GL_LIGHTING );

    int xtex = glutGet ( GLUT_WINDOW_WIDTH ) / _background_image_height;
    int ytex = glutGet ( GLUT_WINDOW_HEIGHT ) / _background_image_width;

    glBegin(GL_POLYGON); {
      glTexCoord2i ( 0, 0 );
      glVertex2f ( -1, -1 );
      glTexCoord2i ( 0, ytex );
      glVertex2f ( -1, 1 );
      glTexCoord2i ( xtex, ytex );
      glVertex2f ( 1, 1 );
      glTexCoord2i ( xtex, 0 );
      glVertex2f ( 1, -1 );
    } glEnd();

    glDisable ( GL_TEXTURE_2D );
    glDisable ( GL_LIGHTING );
  }
}

void htl::hplot::save ( const string & filename, int format  )
{
  FILE * fp = fopen ( filename.c_str(), "wb" );

  int state = GL2PS_OVERFLOW, buffsize = 0;

  // GL2PS does not support textures...
  static bool _gl2ps_background = false;
  std::swap ( _show_background, _gl2ps_background );

  while ( state == GL2PS_OVERFLOW ) {
    buffsize += 1024*1024;
    gl2psBeginPage ( filename.c_str(), 
		     ((string(PACKAGE)+=' ')+=VERSION).c_str(), 
		     0, 
		     format, 
		     GL2PS_BSP_SORT, 
		     GL2PS_USE_CURRENT_VIEWPORT | GL2PS_BEST_ROOT,
		     GL_RGBA, 0, NULL, 
		     8, 8, 8, 
		     buffsize, 
		     fp, 
		     filename.c_str() );
    display();
    state = gl2psEndPage();
  }

  std::swap ( _show_background, _gl2ps_background );

  fclose(fp);
}
