/***************************************************************************
                          htl_plot.h  -  description
                             -------------------
    begin                : Sat Oct 4 2003
    copyright            : (C) 2003 by Soloviev Alexei,JINR,LIT-303,64713
    email                : solovjev@cv.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HTL_PLOT_H
#define HTL_PLOT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "htl_contour.h"
#include "htl_bounding_box.h"

#include "gl2ps.h"

#include <vector>
#include <string>
#include <map>

#include <GL/glut.h>

namespace htl {

class hplot : public bounding_box, public vector<contour>
{
public:
  hplot ( const char* title = PACKAGE,
	  int x = glutGet((GLenum)GLUT_INIT_WINDOW_X),
	  int y = glutGet((GLenum)GLUT_INIT_WINDOW_Y),
	  int width = 640,
	  int height = 480 );
  virtual ~hplot();

public:
  hplot & operator<< ( const contour & c );

  const contour::STYLE & style() const { return _style; }
  contour::STYLE & style() { return _style; }

  void setup();

  virtual void display();
  virtual void keyboard ( unsigned char key, int x, int y );
  virtual void mouse ( int button, int state, int x, int y );
  virtual void motion ( int x, int y );
  virtual void menu ( int option );

protected:
  void unproject ( int xw, int yw, double & x, double & y ) const;

protected:
  contour::STYLE _style;

protected:
  bool _show_background;
  unsigned char* _background_image;
  int _background_image_height;
  int _background_image_width;
  bool load_background_image ( const string & name );
  void display_background();

protected:
  bool _do_mark;
  double _x_min_mark;
  double _x_max_mark;
  double _y_min_mark;
  double _y_max_mark;
  bool _do_move;

public:
  void save ( const string & filename, int format = GL2PS_EPS );

protected:
  static map<int,hplot*> _registry;
  static void _init ();
  static void _display ();
  static void _keyboard ( unsigned char key, int x, int y );
  static void _mouse ( int button, int state, int x, int y );
  static void _motion ( int x, int y );
  static void _menu ( int option );

private:
  int _scale_menu;
  int _style_menu;
  int _decoration_menu;
};

} /** namespace htl */

template<class Tx, class Ty>
htl::hplot & operator<< ( htl::hplot & pout, const htl::histogram<Tx,Ty> & h )
{
  return pout << htl::contour ( h );
}

htl::hplot & operator<< ( htl::hplot & pout, htl::contour::STYLE style );

#endif // HTL_PLOT_H

// Local Variables:
// mode:C++
// End:
