/***************************************************************************
                          htl_text_label.cpp  -  description
                             -------------------
    begin                : Sun Oct 5 2003
    copyright            : (C) 2003 by Soloviev Alexei,JINR,LIT-303,64713
    email                : solovjev@cv.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "htl_text_label.h"
#include "gl2ps.h"

#include <algorithm>
using namespace std;

#include <GL/glut.h>

htl::text_label::text_label ( const string & text ) : 
  _text ( text ), 
  _x ( 0 ), 
  _y ( 0 ),
  _font ( GLUT_BITMAP_HELVETICA_10 ),
  _gl2ps_fontname ( "Helvetica" ),
  _gl2ps_fontsize ( 12 ),
  _x_alignment ( center ),
  _y_alignment ( middle ),
  _x_offset ( 0 ),
  _y_offset ( 0 )
{}

void htl::text_label::plot() const
{
  _x_offset = _text.size();
  for ( unsigned int i = 0; i < _text.size(); ++i ) {
    _x_offset += glutBitmapWidth ( _font, _text[i] );
  }
  _x_offset /= -glutGet ( GLUT_WINDOW_WIDTH );
  _x_offset *= _x_alignment;

  _y_offset = _gl2ps_fontsize;
  _y_offset /= -glutGet ( GLUT_WINDOW_HEIGHT );
  _y_offset *= _y_alignment;

  glMatrixMode ( GL_PROJECTION );
  glPushMatrix();
  glTranslatef ( _x_offset, _y_offset, 0 );

  glMatrixMode ( GL_MODELVIEW );

  glRasterPos2f ( _x, _y );

  /** gl2psText should be called before the glut function */
  gl2psText ( _text.c_str(), _gl2ps_fontname.c_str(), _gl2ps_fontsize );

  for ( string::const_iterator s = _text.begin(); s != _text.end(); ++s )
    glutBitmapCharacter ( _font, *s );

  glMatrixMode ( GL_PROJECTION );
  glPopMatrix();

  glMatrixMode ( GL_MODELVIEW );
}

htl::title::title ( const string & text ) : 
  text_label ( text ) 
{
  _x = 0;
  _y = 1;

  _font = GLUT_BITMAP_HELVETICA_18;
  _gl2ps_fontsize = 18;

  _x_alignment = center;
  _y_alignment = bottom;
}

void htl::title::plot() const
{
  glColor3f ( 0.0f, 0.4f, 0.0f );

  glMatrixMode ( GL_PROJECTION );
  glPushMatrix();
  glTranslatef ( 0, 0.02, 0 );

  glMatrixMode ( GL_MODELVIEW );
  glPushMatrix();
  glLoadIdentity();

  text_label::plot();

  glMatrixMode ( GL_PROJECTION );
  glPopMatrix();

  glMatrixMode ( GL_MODELVIEW );
  glPopMatrix();

  glutPostRedisplay ();
}

htl::x_label::x_label ( const string & text ) : 
  text_label ( text ) 
{
  _x = 1;
  _y = -1;

  _x_alignment = right;
  _y_alignment = top;
}

void htl::x_label::plot() const
{
  glColor3f ( 0.0f, 0.0f, 0.6f );

  glMatrixMode ( GL_PROJECTION );
  glPushMatrix();
  glTranslatef ( 0, _y_offset, 0 );

  glMatrixMode ( GL_MODELVIEW );
  glPushMatrix();
  glLoadIdentity();

  text_label::plot();

  glMatrixMode ( GL_PROJECTION );
  glPopMatrix();

  glMatrixMode ( GL_MODELVIEW );
  glPopMatrix();

  glutPostRedisplay ();
}

htl::y_label::y_label ( const string & text ) : 
  text_label ( text ) 
{
  _x = -1;
  _y = 1;

  _x_alignment = right;
  _y_alignment = middle;
}

void htl::y_label::plot() const
{
  glColor3f ( 0.0f, 0.0f, 0.6f );

  glMatrixMode ( GL_MODELVIEW );
  glPushMatrix();
  glLoadIdentity();

  text_label::plot();

  glPopMatrix();

  glutPostRedisplay ();
}
