/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Sun Oct 19 2003
    copyright            : (C) 2003 by Soloviev Alexei,JINR,LIT-303,64713
    email                : solovjev@cv.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "getopt.h"

#include <histogram>
#include <hplot>
using namespace htl;

#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <vector>
#include <string>
#include <map>
#include <algorithm>
using namespace std;

static struct option long_options[] = {
  {"title", 1, 0, 't'},
  {"x-label", 1, 0, 'x'},
  {"y-label", 1, 0, 'y'},
  {"using", 1, 0, 'u'},
  {"with", 1, 0, 'w'},
  {"output", 1, 0, 'o'},
  {"help", 0, 0, 'h'},
  {"version", 0, 0, 'V'},
  {0, 0, 0, 0}
};

typedef map < string, string > Section;
typedef map < string, Section > Book;

Book & book()
{
  static Book fBook;

  if ( fBook.empty() ) {

    static Section fCommon;
    if ( fCommon.empty() ) {
      fCommon["title"] = "set plot title [none]";
      fCommon["x-label"] = "set x-axis label [none]";
      fCommon["y-label"] = "set y-axis label [none]";
      fCommon["SectionArgument"] = "string";
    }
    fBook["Common"] = fCommon;

    static Section fPlot;
    if ( fPlot.empty() ) {
      fPlot["using"] = "set columns in file for plot [1:2]";
      fPlot["with"] = "set plot style (see \'--with help\' for list) [lines]";
      fPlot["SectionArgument"] = "string";
    }
    fBook["Plot"] = fPlot;

    static Section fOutput;
    if ( fOutput.empty() ) {
      fOutput["output"] = "output plot to file in EPS format [GLUT window]";
      fOutput["SectionArgument"] = "file";
    }
    fBook["Output"] = fOutput;

    static Section fHelp;
    if ( fHelp.empty() ) {
      fHelp["help"] = "display this help and exit";
      fHelp["version"] = "display version information and exit";
    }
    fBook["Help"] = fHelp;

  }
  return fBook;
}

void usage ( ostream & out = cerr )
{
  out << "Usage: " << PACKAGE << " [options] file(s)" << endl;
  out << "Options: [defaults in brackets after descriptions]" << endl;

  static const int opt_width = 26;
  out.setf(ios::left,ios::adjustfield);

  Book::const_iterator sect = book().begin();
  while ( sect != book().end() ) {
    out << sect->first << " options:" << endl;
    option * optptr = long_options;
    while ( optptr->name != 0 ) {
      if ( book()[sect->first].find( optptr->name ) !=
	   book()[sect->first].end() ) {
	string optstr = "  ";
	if ( optptr->val && isalpha ( optptr->val ) ) {
	  ( ( optstr += "-" ) += optptr->val ) += ", ";
	}
	( optstr += "--" ) += optptr->name;
	if ( optptr->has_arg ) {
	  string arg = sect->first;
	  if ( book()[sect->first].find( "SectionArgument" ) !=
	       book()[sect->first].end() ) {
	    arg = book()[sect->first]["SectionArgument"];
	  }
	  transform ( arg.begin(), arg.end(), arg.begin(), ::tolower );
	  if ( optptr->has_arg == 2 ) ( ( optstr += " [" ) += arg ) += "] ";
	  else ( ( optstr += " <" ) += arg ) += "> ";
	}
	out << setw(opt_width) << optstr.c_str();
	out << book()[sect->first][optptr->name] << endl;
      }
      ++optptr;
    }
    ++sect;
  }
}

int main ( int argc, char** argv )
{
  if ( argc == 1 ) {
    cout << PACKAGE << ": missing arguments" << endl;
    cout << "Try \"" << PACKAGE << " --help\" for more information." << endl;
    exit(1);
  }

  hplot pout ( PACKAGE );
  string output;

  string optstring;
  option * optptr = long_options;
  while ( optptr->name != 0 ) {
    if ( optptr->val && isalpha ( optptr->val ) ) {
      optstring += optptr->val;
      if ( optptr->has_arg == 1 ) optstring += ':';
      if ( optptr->has_arg == 2 ) optstring += '?';
    }
    ++optptr;
  }

  int c = 0;
  int option_index = 0;

  // getopt first pass: common options
  char* argv1[argc];
  copy ( argv, argv+argc, argv1 );
  while ( c != -1 ) {
    c = getopt_long ( argc, argv1, 
		      optstring.c_str(), long_options, 
		      &option_index );
    switch (c) {
    case 't': pout.set_title(optarg); break;
    case 'x': pout.set_x_label(optarg); break;
    case 'y': pout.set_y_label(optarg); break;

    case 'o': output = optarg; break;

    case 'V':
      cout << PACKAGE << " version " << VERSION << endl;
      exit(0);
      break;

    case 'h':
      usage();
      exit(0);
      break;

    default:
      break;
    }
  }

  if ( optind == argc ) {
    cerr << PACKAGE << ": no input file(s) specified"<< endl;
    exit(1);
  }

  int nfiles = argc - optind;
  int filepos[nfiles];
  for ( int i = 0; i < nfiles; ++i )
    filepos[i] = find ( argv, argv+argc, argv1[optind++] ) - argv;

  // getopt second pass: plot options
  for ( int i = 0; i < nfiles; ++i ) {
    int xcol = 1, ycol = 2;
    contour::STYLE style = contour::lines;
    int shift = i==0 ? 0 : filepos[i-1];
    c = 0;
    option_index = optind = 0;
    while ( c != -1 ) {
      c = getopt_long ( filepos[i]-shift, argv+shift, 
			optstring.c_str(), long_options, 
			&option_index );
      switch (c) {
      case 'u':
	{
	  string arg ( optarg );
	  string::iterator p = find ( arg.begin(), arg.end(), ':' );
	  if ( p == arg.end() ) {
	    cerr << PACKAGE << ": using range is invalid"<< endl;
	    exit(1);
	  }
	  string x_str ( arg.begin(), p );
	  string y_str ( ++p, arg.end() );
	  if ( !x_str.empty() ) xcol = atoi ( x_str.c_str() );
	  if ( !y_str.empty() ) ycol = atoi ( y_str.c_str() );
	  if ( xcol > ycol ) {
	    swap ( xcol, ycol );
	    cerr << PACKAGE << ": --using " << xcol << ":" << ycol << " assumed" << endl;
	  }
	}
	break;

      case 'w':
	{
	  static const string stylestr[] = 
	  { "lines",
	    "points",
	    "linespoints",
	    "dots",
	    "impulses",
	    "steps",
	    "boxes",
	    "candlesticks",
	    "errorbars",
	    "help"
	  };
	  int s = 0;
	  string optargstr = optarg;
	  while ( stylestr[s] != "help" ) {
	    if ( search ( stylestr[s].begin(), stylestr[s].end(), 
			  optargstr.begin(), optargstr.end() ) 
		 == stylestr[s].begin() ) {
	      style = static_cast<contour::STYLE>(s);
	      break;
	    }
	    ++s;
	  }
  	  if ( stylestr[s] == "help" ) {
	    cout << PACKAGE << ": --with ";
	    s = 0;
	    while ( stylestr[s] != "help" ) {
	      cout << "\'" << stylestr[s] << "\'";
	      if ( stylestr[++s] != "help" ) cout << ", ";
	    }
	    cout << " expecting" << endl;
	    exit(1);
	  }
	}
	break;

      default:
	break;
      }
    }

    ifstream in ( argv[filepos[i]] );
    if ( !in.is_open() ) {
      cerr << PACKAGE << ": cannot open input file \""
	   << argv[filepos[i]] << "\"" << endl;
      exit(1);
    }

    vector<double> x, y;
    static char buff[1024];
    while ( in.getline ( buff, 1024, '\n' ) ) {
      istringstream line ( buff );
      istream_iterator<double> iter ( line );
      istream_iterator<double> eof;

      for ( int i = 1; i < xcol; ++i ) ++iter;
      if ( iter == eof ) {
	cerr << PACKAGE << ": using range is invalid"<< endl;
	exit(1);
      }
      x.push_back ( *iter++ );
      for ( int i = xcol+1; i < ycol; ++i ) ++iter;
      if ( iter == eof ) {
	cerr << PACKAGE << ": using range is invalid"<< endl;
	exit(1);
      }
      y.push_back ( *iter++ );
    }
  
    unsigned int n = x.size();

    vector<double> dx(n);
    dx[0] = x[1] - x[0];
    for ( unsigned int i = 1; i < n; ++i ) {
      dx[i] = x[i] - x[i-1];
    }

    double xmin = x[0] - dx[0]/2;
    double xmax = x[n-1] + dx[n-1]/2;

    domain<double> axis ( xmin, xmax, n, dx.begin() );
    histogram<double,double> h ( axis );
    copy ( y.begin(), y.end(), h.begin() );

    pout << style << h;
  }

  if ( !output.empty() ) {
    pout.save(output);
    exit(0);
  }

  return 0;
}
